@@ -1,4 +1,4 @@
-/* Copyright (c) 2013. The SimGrid Team.
+/* Copyright (c) 2013-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -32,7 +32,7 @@ int main(int argc, char **argv)
   total_nworkstations = SD_workstation_get_number();
 
   for (ctr=0; ctr<total_nworkstations;ctr++){
-    current_storage_list = SD_workstation_get_storage_list(workstations[ctr]);
+    current_storage_list = SD_workstation_get_mounted_storage_list(workstations[ctr]);
     xbt_dict_foreach(current_storage_list,cursor,mount_name,storage_name)
       XBT_INFO("Workstation '%s' mounts '%s'",
          SD_workstation_get_name(workstations[ctr]), mount_name);
