@@ -1,6 +1,6 @@
 /* A simple bugged MPI_ISend and MPI_IRecv test */
 
-/* Copyright (c) 2009, 2011, 2013. The SimGrid Team.
+/* Copyright (c) 2009, 2011, 2013-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -23,8 +23,8 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-  err = MPI_Comm_size(MPI_COMM_WORLD, &size);   /* Get nr of tasks */
-  err = MPI_Comm_rank(MPI_COMM_WORLD, &rank);   /* Get id of this process */
+  MPI_Comm_size(MPI_COMM_WORLD, &size);   /* Get nr of tasks */
+  MPI_Comm_rank(MPI_COMM_WORLD, &rank);   /* Get id of this process */
   if (size < 2) {
     printf("run this program with exactly 2 processes (-np 2)\n");
     MPI_Finalize();
