@@ -1,16 +1,14 @@
-/*
- * jedule_sd_binding.h
- *
- *  Created on: Dec 2, 2010
- *      Author: sascha
- */
+/* Copyright (c) 2010-2011, 2013-2014. The SimGrid Team.
+ * All rights reserved.                                                     */
+
+/* This program is free software; you can redistribute it and/or modify it
+ * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #ifndef JEDULE_SD_BINDING_H_
 #define JEDULE_SD_BINDING_H_
 
 #include "simgrid_config.h"
 
-#include "simdag/private.h"
 #include "simdag/datatypes.h"
 #include "simdag/simdag.h"
 
