@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2010, 2012-2013. The SimGrid Team.
+/* Copyright (c) 2006-2010, 2012-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -83,6 +83,7 @@ XBT_PUBLIC(int) SD_route_get_size(SD_workstation_t src,
 XBT_PUBLIC(double) SD_workstation_get_power(SD_workstation_t workstation);
 XBT_PUBLIC(double) SD_workstation_get_available_power(SD_workstation_t
                                                       workstation);
+XBT_PUBLIC(int) SD_workstation_get_cores(SD_workstation_t workstation);
 XBT_PUBLIC(e_SD_workstation_access_mode_t)
     SD_workstation_get_access_mode(SD_workstation_t workstation);
 XBT_PUBLIC(void) SD_workstation_set_access_mode(SD_workstation_t
@@ -106,7 +107,10 @@ XBT_PUBLIC(double) SD_route_get_communication_time(SD_workstation_t src,
 XBT_PUBLIC(SD_task_t) SD_workstation_get_current_task(SD_workstation_t
                                                       workstation);
 XBT_PUBLIC(xbt_dict_t)
-    SD_workstation_get_storage_list(SD_workstation_t workstation);
+    SD_workstation_get_mounted_storage_list(SD_workstation_t workstation);
+XBT_PUBLIC(xbt_dynar_t)
+    SD_workstation_get_attached_storage_list(SD_workstation_t workstation);
+XBT_PUBLIC(const char*) SD_storage_get_host(SD_storage_t storage);
 /** @} */
 
 /************************** Task handling ************************************/
