@@ -1,9 +1,10 @@
 /* Functions related to the java file API.                            */
-/* Copyright (c) 2012-2013. The SimGrid Team.
+
+/* Copyright (c) 2012-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
-  * under the terms of the license (GNU LGPL) which comes with this package. */
+ * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #ifndef MSG_JFILE_H
 #define MSG_JFILE_H
@@ -27,7 +28,7 @@ Java_org_simgrid_msg_File_nativeInit(JNIEnv*, jclass);
  * Signature	(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
  */
 JNIEXPORT void JNICALL
-Java_org_simgrid_msg_File_open(JNIEnv*, jobject, jobject, jobject);
+Java_org_simgrid_msg_File_open(JNIEnv*, jobject, jobject);
 /**
  * Class 			org_simgrid_msg_File
  * Method			read
