@@ -1,6 +1,6 @@
 /* platf_interface.h - Internal interface to the SimGrid platforms          */
 
-/* Copyright (c) 2004-2007, 2009-2012. The SimGrid Team.
+/* Copyright (c) 2004-2007, 2009-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -12,6 +12,8 @@
 #include "simgrid/platf.h" /* public interface */
 #include "xbt/RngStream.h"
 
+SG_BEGIN_DECL()
+
 /* Module management functions */
 XBT_PUBLIC(void) sg_platf_init(void);
 XBT_PUBLIC(void) sg_platf_exit(void);
@@ -80,4 +82,6 @@ XBT_PUBLIC(void) surf_config_models_setup(void);
 XBT_PUBLIC(void) sg_platf_rng_stream_init(unsigned long seed[6]);
 XBT_PUBLIC(RngStream) sg_platf_rng_stream_get(const char* id);
 
+SG_END_DECL()
+
 #endif                          /* SG_PLATF_INTERFACE_H */
