@@ -1,4 +1,4 @@
-/* Copyright (c) 2012-2013. The SimGrid Team.
+/* Copyright (c) 2012-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -9,9 +9,13 @@
 /*******************************************/
 /*** Config Globals **************************/
 /*******************************************/
+
+SG_BEGIN_DECL()
+
 XBT_PUBLIC_DATA(xbt_cfg_t) _sg_cfg_set;
 XBT_PUBLIC_DATA(int) _sg_cfg_init_status;
 XBT_PUBLIC_DATA(int) _sg_cfg_exit_asap;
+XBT_PUBLIC(int) sg_cfg_is_default_value(const char* name);
 XBT_PUBLIC(int) sg_cfg_get_int(const char* name);
 XBT_PUBLIC(double) sg_cfg_get_double(const char* name);
 XBT_PUBLIC(char*) sg_cfg_get_string(const char* name);
@@ -21,3 +25,5 @@ XBT_PUBLIC(xbt_dynar_t) sg_cfg_get_dynar(const char* name);
 
 void sg_config_init(int *argc, char **argv);
 void sg_config_finalize(void);
+
+SG_END_DECL()
