@@ -1,8 +1,8 @@
-/* Copyright (c) 2010-2013. The SimGrid Team.
+/* Copyright (c) 2010-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
-  * under the terms of the license (GNU LGPL) which comes with this package. */
+ * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #ifndef INSTR_PRIVATE_H_
 #define INSTR_PRIVATE_H_
@@ -14,7 +14,10 @@
 
 #ifdef HAVE_TRACING
 
+SG_BEGIN_DECL()
+
 /* Need to define function drand48 for Windows */
+/* FIXME: use _drand48() defined in src/surf/random_mgr.c instead */
 #ifdef _WIN32
 #  define drand48() (rand()/(RAND_MAX + 1.0))
 #endif
@@ -295,6 +298,7 @@ void instr_resume_tracing (void);
 XBT_PUBLIC(int) TRACE_smpi_is_enabled(void);
 XBT_PUBLIC(int) TRACE_smpi_is_grouped(void);
 XBT_PUBLIC(int) TRACE_smpi_is_computing(void);
+XBT_PUBLIC(int) TRACE_smpi_is_sleeping(void);
 XBT_PUBLIC(int) TRACE_smpi_view_internals(void);
 
 /* from resource_utilization.c */
@@ -345,7 +349,6 @@ XBT_PUBLIC(val_t)  PJ_value_get_or_new (const char *name, const char *color, typ
 XBT_PUBLIC(val_t)  PJ_value_get (const char *name, const type_t father);
 void PJ_value_free (val_t value);
 
-
 void print_pajeDefineContainerType(paje_event_t event);
 void print_pajeDefineVariableType(paje_event_t event);
 void print_pajeDefineStateType(paje_event_t event);
@@ -420,6 +423,7 @@ typedef enum{
   TRACING_RECV,
   TRACING_IRECV,
   TRACING_SENDRECV,
+  TRACING_TEST,
   TRACING_WAIT,
   TRACING_WAITALL,
   TRACING_WAITANY,
@@ -437,6 +441,7 @@ typedef enum{
   TRACING_ALLGATHERV,
   TRACING_REDUCE_SCATTER,
   TRACING_COMPUTING,
+  TRACING_SLEEPING,
   TRACING_SCAN,
   TRACING_EXSCAN
 } e_caller_type ;
@@ -448,6 +453,7 @@ typedef struct s_instr_extra_data {
   int send_size;
   int recv_size;
   double comp_size;
+  double sleep_duration;
   int src;
   int dst;
   int root;
@@ -458,15 +464,12 @@ typedef struct s_instr_extra_data {
   int num_processes;
 } s_instr_extra_data_t;
 
+SG_END_DECL()
+
 #endif /* HAVE_TRACING */
 
 #ifdef HAVE_JEDULE
 #include "instr/jedule/jedule_sd_binding.h"
 #endif
 
-
-
-
-
-
 #endif /* INSTR_PRIVATE_H_ */
