@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2013. The SimGrid Team.
+/* Copyright (c) 2007-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@ XBT_INLINE smx_process_t SIMIX_process_self(void)
 {
   smx_context_t self_context = SIMIX_context_self();
 
-  return self_context ? SIMIX_context_get_data(self_context) : NULL;
+  return self_context ? SIMIX_context_get_process(self_context) : NULL;
 }
 
 /**
@@ -49,6 +49,8 @@ void SIMIX_process_cleanup(smx_process_t process)
   XBT_DEBUG("Cleanup process %s (%p), waiting action %p",
       process->name, process, process->waiting_action);
 
+  SIMIX_process_on_exit_runall(process);
+
   /* cancel non-blocking communications */
   smx_action_t action;
   while ((action = xbt_fifo_pop(process->comms))) {
@@ -101,7 +103,7 @@ void SIMIX_process_cleanup(smx_process_t process)
   process->context->iwannadie = 0;
 }
 
-/** 
+/**
  * Garbage collection
  *
  * Should be called some time to time to free the memory allocated for processes
@@ -137,18 +139,12 @@ void SIMIX_create_maestro_process()
   /* Create maestro process and intilialize it */
   maestro = xbt_new0(s_smx_process_t, 1);
   maestro->pid = simix_process_maxpid++;
+  maestro->ppid = -1;
   maestro->name = (char *) "";
   maestro->running_ctx = xbt_new(xbt_running_ctx_t, 1);
   XBT_RUNNING_CTX_INITIALIZE(maestro->running_ctx);
   maestro->context = SIMIX_context_new(NULL, 0, NULL, NULL, maestro);
   maestro->simcall.issuer = maestro;
-
-  if (SIMIX_process_self()) {
-    maestro->ppid = SIMIX_process_get_PID(SIMIX_process_self());
-  } else {
-    maestro->ppid = -1;
-  }
-
   simix_global->maestro_process = maestro;
   return;
 }
@@ -185,17 +181,17 @@ void SIMIX_process_stop(smx_process_t arg) {
 smx_process_t SIMIX_process_create_from_wrapper(smx_process_arg_t args) {
 
   smx_process_t process;
-  simix_global->create_process_function(
-      &process,
-      args->name,
-      args->code,
-      args->data,
-      args->hostname,
-      args->kill_time,
-      args->argc,
-      args->argv,
-      args->properties,
-      args->auto_restart);
+  simix_global->create_process_function(&process,
+                                        args->name,
+                                        args->code,
+                                        args->data,
+                                        args->hostname,
+                                        args->kill_time,
+                                        args->argc,
+                                        args->argv,
+                                        args->properties,
+                                        args->auto_restart,
+                                        NULL);
   xbt_free(args);
   return process;
 }
@@ -211,8 +207,9 @@ void SIMIX_pre_process_create(smx_simcall_t simcall,
                           int argc, char **argv,
                           xbt_dict_t properties,
                           int auto_restart){
-  return SIMIX_process_create(process, name, code, data, hostname,
-                              kill_time, argc, argv, properties, auto_restart);
+  SIMIX_process_create(process, name, code, data, hostname,
+                       kill_time, argc, argv, properties, auto_restart,
+                       simcall->issuer);
 }
 /**
  * \brief Internal function to create a process.
@@ -231,8 +228,9 @@ void SIMIX_process_create(smx_process_t *process,
                           double kill_time,
                           int argc, char **argv,
                           xbt_dict_t properties,
-                          int auto_restart) {
-
+                          int auto_restart,
+                          smx_process_t parent_process)
+{
   *process = NULL;
   smx_host_t host = SIMIX_host_get_by_name(hostname);
 
@@ -257,9 +255,9 @@ void SIMIX_process_create(smx_process_t *process,
     (*process)->data = data;
     (*process)->comms = xbt_fifo_new();
     (*process)->simcall.issuer = *process;
-    
-     if (SIMIX_process_self()) {
-       (*process)->ppid = SIMIX_process_get_PID(SIMIX_process_self());
+
+     if (parent_process) {
+       (*process)->ppid = SIMIX_process_get_PID(parent_process);
      } else {
        (*process)->ppid = -1;
      }
@@ -295,10 +293,8 @@ void SIMIX_process_create(smx_process_t *process,
     xbt_swag_insert(*process, simix_global->process_list);
     XBT_DEBUG("Inserting %s(%s) in the to_run list", (*process)->name, sg_host_name(host));
     xbt_dynar_push_as(simix_global->process_to_run, smx_process_t, *process);
-  }
 
-  if (kill_time > SIMIX_get_clock()) {
-    if (simix_global->kill_process_function) {
+    if (kill_time > SIMIX_get_clock() && simix_global->kill_process_function) {
       XBT_DEBUG("Process %s(%s) will be kill at time %f", (*process)->name,
           sg_host_name((*process)->smx_host), kill_time);
       SIMIX_timer_set(kill_time, simix_global->kill_process_function, *process);
@@ -366,6 +362,10 @@ void SIMIX_process_kill(smx_process_t process, smx_process_t issuer) {
       SIMIX_process_sleep_destroy(process->waiting_action);
       break;
 
+    case SIMIX_ACTION_JOIN:
+      SIMIX_process_sleep_destroy(process->waiting_action);
+      break;
+
     case SIMIX_ACTION_SYNCHRO:
       SIMIX_synchro_stop_waiting(process, &process->simcall);
       SIMIX_synchro_destroy(process->waiting_action);
@@ -491,7 +491,8 @@ smx_action_t SIMIX_process_suspend(smx_process_t process, smx_process_t issuer)
       return NULL;
     }
   } else {
-    return SIMIX_host_execute("suspend", process->smx_host, 0.0, 1.0);
+    /* FIXME: computation size is zero. Is it okay that bound is zero ? */
+    return SIMIX_host_execute("suspend", process->smx_host, 0.0, 1.0, 0.0, 0);
   }
 }
 
@@ -521,7 +522,7 @@ void SIMIX_process_resume(smx_process_t process, smx_process_t issuer)
 
       switch (process->waiting_action->type) {
 
-        case SIMIX_ACTION_EXECUTE:          
+        case SIMIX_ACTION_EXECUTE:
         case SIMIX_ACTION_PARALLEL_EXECUTE:
           SIMIX_host_execution_resume(process->waiting_action);
           break;
@@ -584,7 +585,7 @@ int SIMIX_process_get_PPID(smx_process_t self){
 }
 
 void* SIMIX_pre_process_self_get_data(smx_simcall_t simcall, smx_process_t self){
-  return SIMIX_process_self_get_data(self);	
+  return SIMIX_process_self_get_data(self);
 }
 
 void* SIMIX_process_self_get_data(smx_process_t self)
@@ -676,6 +677,44 @@ xbt_dict_t SIMIX_process_get_properties(smx_process_t process)
   return process->properties;
 }
 
+void SIMIX_pre_process_join(smx_simcall_t simcall, smx_process_t process, double timeout)
+{
+  smx_action_t action = SIMIX_process_join(simcall->issuer, process, timeout);
+  xbt_fifo_push(action->simcalls, simcall);
+  simcall->issuer->waiting_action = action;
+}
+
+static int SIMIX_process_join_finish(smx_process_exit_status_t status, smx_action_t action){
+  if (action->sleep.surf_sleep) {
+    surf_action_cancel(action->sleep.surf_sleep);
+
+    smx_simcall_t simcall;
+    while ((simcall = xbt_fifo_shift(action->simcalls))) {
+      simcall_process_sleep__set__result(simcall, SIMIX_DONE);
+      simcall->issuer->waiting_action = NULL;
+      if (simcall->issuer->suspended) {
+        XBT_DEBUG("Wait! This process is suspended and can't wake up now.");
+        simcall->issuer->suspended = 0;
+        SIMIX_pre_process_suspend(simcall, simcall->issuer);
+      } else {
+        SIMIX_simcall_answer(simcall);
+      }
+    }
+    surf_action_unref(action->sleep.surf_sleep);
+    action->sleep.surf_sleep = NULL;
+  }
+  xbt_mallocator_release(simix_global->action_mallocator, action);
+  return 0;
+}
+
+smx_action_t SIMIX_process_join(smx_process_t issuer, smx_process_t process, double timeout)
+{
+  smx_action_t res = SIMIX_process_sleep(issuer, timeout);
+  res->type = SIMIX_ACTION_JOIN;
+  SIMIX_process_on_exit(process, (int_f_pvoid_pvoid_t)SIMIX_process_join_finish, res);
+  return res;
+}
+
 void SIMIX_pre_process_sleep(smx_simcall_t simcall, double duration)
 {
   if (MC_is_active()) {
@@ -695,8 +734,7 @@ smx_action_t SIMIX_process_sleep(smx_process_t process, double duration)
   smx_host_t host = process->smx_host;
 
   /* check if the host is active */
-  if (surf_workstation_model->extension.
-      workstation.get_state(host) != SURF_RESOURCE_ON) {
+  if (surf_resource_get_state(surf_workstation_resource_priv(host)) != SURF_RESOURCE_ON) {
     THROWF(host_error, 0, "Host %s failed, you cannot call this function",
            sg_host_name(host));
   }
@@ -710,9 +748,9 @@ smx_action_t SIMIX_process_sleep(smx_process_t process, double duration)
 
   action->sleep.host = host;
   action->sleep.surf_sleep =
-      surf_workstation_model->extension.workstation.sleep(host, duration);
+      surf_workstation_sleep(host, duration);
 
-  surf_workstation_model->action_data_set(action->sleep.surf_sleep, action);
+  surf_action_set_data(action->sleep.surf_sleep, action);
   XBT_DEBUG("Create sleep action %p", action);
 
   return action;
@@ -722,10 +760,11 @@ void SIMIX_post_process_sleep(smx_action_t action)
 {
   smx_simcall_t simcall;
   e_smx_state_t state;
+  xbt_assert(action->type == SIMIX_ACTION_SLEEP || action->type == SIMIX_ACTION_JOIN);
 
   while ((simcall = xbt_fifo_shift(action->simcalls))) {
 
-    switch(surf_workstation_model->action_state_get(action->sleep.surf_sleep)){
+    switch(surf_action_get_state(action->sleep.surf_sleep)){
       case SURF_ACTION_FAILED:
         simcall->issuer->context->iwannadie = 1;
         //SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
@@ -740,37 +779,50 @@ void SIMIX_post_process_sleep(smx_action_t action)
         THROW_IMPOSSIBLE;
         break;
     }
-    if (surf_workstation_model->extension.
-        workstation.get_state(simcall->issuer->smx_host) != SURF_RESOURCE_ON) {
+    if (surf_resource_get_state(surf_workstation_resource_priv(simcall->issuer->smx_host)) != SURF_RESOURCE_ON) {
       simcall->issuer->context->iwannadie = 1;
     }
     simcall_process_sleep__set__result(simcall, state);
     simcall->issuer->waiting_action = NULL;
-    SIMIX_simcall_answer(simcall);
-
+    if (simcall->issuer->suspended) {
+      XBT_DEBUG("Wait! This process is suspended and can't wake up now.");
+      simcall->issuer->suspended = 0;
+      SIMIX_pre_process_suspend(simcall, simcall->issuer);
+    } else {
+      SIMIX_simcall_answer(simcall);
+    }
   }
+
   SIMIX_process_sleep_destroy(action);
 }
 
 void SIMIX_process_sleep_destroy(smx_action_t action)
 {
   XBT_DEBUG("Destroy action %p", action);
-  if (action->sleep.surf_sleep)
-    action->sleep.surf_sleep->model_type->action_unref(action->sleep.surf_sleep);
-  xbt_mallocator_release(simix_global->action_mallocator, action);
+  xbt_assert(action->type == SIMIX_ACTION_SLEEP || action->type == SIMIX_ACTION_JOIN);
+
+  if (action->sleep.surf_sleep) {
+    surf_action_unref(action->sleep.surf_sleep);
+    action->sleep.surf_sleep = NULL;
+  }
+  if (action->type == SIMIX_ACTION_SLEEP)
+    xbt_mallocator_release(simix_global->action_mallocator, action);
 }
 
 void SIMIX_process_sleep_suspend(smx_action_t action)
 {
-  surf_workstation_model->suspend(action->sleep.surf_sleep);
+  xbt_assert(action->type == SIMIX_ACTION_SLEEP);
+  surf_action_suspend(action->sleep.surf_sleep);
 }
 
 void SIMIX_process_sleep_resume(smx_action_t action)
 {
-  surf_workstation_model->resume(action->sleep.surf_sleep);
+  XBT_DEBUG("Action state is %d on process_sleep_resume.", action->state);
+  xbt_assert(action->type == SIMIX_ACTION_SLEEP);
+  surf_action_resume(action->sleep.surf_sleep);
 }
 
-/** 
+/**
  * \brief Calling this function makes the process to yield.
  *
  * Only the current process can call this function, giving back the control to
@@ -798,10 +850,11 @@ void SIMIX_process_yield(smx_process_t self)
     SIMIX_process_stop(self);
   }
 
-  if(self->suspended) {
+  if (self->suspended) {
+    XBT_DEBUG("Hey! I'm suspended.");
     xbt_assert(!self->doexception, "Gloups! This exception may be lost by subsequent calls.");
     self->suspended = 0;
-    SIMIX_process_suspend(self,self);
+    SIMIX_process_suspend(self, self);
   }
 
   if (self->doexception) {
@@ -868,19 +921,20 @@ xbt_dynar_t SIMIX_processes_as_dynar(void) {
 
 void SIMIX_process_on_exit_runall(smx_process_t process) {
   s_smx_process_exit_fun_t exit_fun;
-
+  smx_process_exit_status_t exit_status = (process->context->iwannadie) ?
+                                         SMX_EXIT_FAILURE : SMX_EXIT_SUCCESS;
   while (!xbt_dynar_is_empty(process->on_exit)) {
     exit_fun = xbt_dynar_pop_as(process->on_exit,s_smx_process_exit_fun_t);
-    (exit_fun.fun)(exit_fun.arg);
+    (exit_fun.fun)((void*)exit_status, exit_fun.arg);
   }
 }
 
 void SIMIX_pre_process_on_exit(smx_simcall_t simcall, smx_process_t process,
-	                       int_f_pvoid_t fun, void *data) {
+	                       int_f_pvoid_pvoid_t fun, void *data) {
   SIMIX_process_on_exit(process, fun, data);
 }
 
-void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data) {
+void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_pvoid_t fun, void *data) {
   xbt_assert(process, "current process not found: are you in maestro context ?");
 
   if (!process->on_exit) {
@@ -894,7 +948,7 @@ void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data)
 
 void SIMIX_pre_process_auto_restart_set(smx_simcall_t simcall, smx_process_t process,
 		                        int auto_restart) {
-  SIMIX_process_auto_restart_set(process, auto_restart);	
+  SIMIX_process_auto_restart_set(process, auto_restart);
 }
 /**
  * \brief Sets the auto-restart status of the process.
@@ -906,7 +960,7 @@ void SIMIX_process_auto_restart_set(smx_process_t process, int auto_restart) {
 }
 
 smx_process_t SIMIX_pre_process_restart(smx_simcall_t simcall, smx_process_t process) {
-  return SIMIX_process_restart(process, simcall->issuer);	
+  return SIMIX_process_restart(process, simcall->issuer);
 }
 /**
  * \brief Restart a process.
@@ -944,19 +998,19 @@ smx_process_t SIMIX_process_restart(smx_process_t process, smx_process_t issuer)
                                           arg.argc,
                                           arg.argv,
                                           arg.properties,
-                                          arg.auto_restart);
-  }
-  else {
+                                          arg.auto_restart,
+                                          NULL);
+  } else {
     simcall_process_create(&new_process,
-                                          arg.argv[0],
-                                          arg.code,
-                                          arg.data,
-                                          arg.hostname,
-                                          arg.kill_time,
-                                          arg.argc,
-                                          arg.argv,
-                                          arg.properties,
-                                          arg.auto_restart);
+                           arg.argv[0],
+                           arg.code,
+                           arg.data,
+                           arg.hostname,
+                           arg.kill_time,
+                           arg.argc,
+                           arg.argv,
+                           arg.properties,
+                           arg.auto_restart);
 
   }
   return new_process;
