@@ -1,8 +1,8 @@
-/* Copyright (c) 2010, 2012-2013. The SimGrid Team.
+/* Copyright (c) 2010, 2012-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
-  * under the terms of the license (GNU LGPL) which comes with this package. */
+ * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "private.h"
 #include <ctype.h>
@@ -26,6 +26,7 @@ static const char *smpi_colors[] ={
     "wait",     "1 1 0",
     "waitall",  "0.78 0.78 0",
     "waitany",  "0.78 0.78 0.58",
+    "test",     "0.52 0.52 0",
 
     "allgather",     "1 0 0",
     "allgatherv",    "1 0.52 0.52",
@@ -42,7 +43,10 @@ static const char *smpi_colors[] ={
     "exscan",          "1 0.54 0.25",
     "scatterv",      "0.52 0 0.52",
     "scatter",       "1 0.74 0.54",
+
     "computing",     "0 1 1",
+    "sleeping",      "0 0.5 0.5",
+
     "init",       "0 1 0",
     "finalize",     "0 1 0",
     NULL, NULL,
@@ -79,16 +83,28 @@ static char *smpi_container(int rank, char *container, int n)
   return container;
 }
 
+static char *TRACE_smpi_get_key(int src, int dst, char *key, int n);
+
+
 static char *TRACE_smpi_put_key(int src, int dst, char *key, int n)
 {
   //get the dynar for src#dst
   char aux[INSTR_DEFAULT_STR_SIZE];
   snprintf(aux, INSTR_DEFAULT_STR_SIZE, "%d#%d", src, dst);
   xbt_dynar_t d = xbt_dict_get_or_null(keys, aux);
+
+
+  if(!xbt_dynar_is_empty(d)){
+    //receive was already pushed, perform a get instead
+    TRACE_smpi_get_key(src , dst, key ,n);
+    return key;
+  }
+
   if (d == NULL) {
     d = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
     xbt_dict_set(keys, aux, d, NULL);
   }
+
   //generate the key
   static unsigned long long counter = 0;
 
@@ -107,8 +123,15 @@ static char *TRACE_smpi_get_key(int src, int dst, char *key, int n)
   snprintf(aux, INSTR_DEFAULT_STR_SIZE, "%d#%d", src, dst);
   xbt_dynar_t d = xbt_dict_get_or_null(keys, aux);
 
-  xbt_assert(!xbt_dynar_is_empty(d),
-      "Trying to get a link key (for message reception) that has no corresponding send (%s).", __FUNCTION__);
+ // xbt_assert(!xbt_dynar_is_empty(d),
+ //     "Trying to get a link key (for message reception) that has no corresponding send (%s).", __FUNCTION__);
+
+  // sometimes the receive may be posted before the send
+  if(xbt_dynar_is_empty(d)){
+      TRACE_smpi_put_key(src, dst, key, n);
+      return key;
+  }
+
   char *s = xbt_dynar_get_as (d, 0, char *);
   snprintf (key, n, "%s", s);
   xbt_dynar_remove_at (d, 0, NULL);
@@ -262,6 +285,75 @@ void TRACE_smpi_computing_out(int rank)
   new_pajePopState (SIMIX_get_clock(), container, type);
 }
 
+void TRACE_smpi_sleeping_init(int rank)
+{
+ //first use, initialize the color in the trace
+ //TODO : check with lucas and Pierre how to generalize this approach
+  //to avoid unnecessary access to the color array
+  if (!TRACE_smpi_is_enabled() || !TRACE_smpi_is_sleeping()) return;
+
+  char str[INSTR_DEFAULT_STR_SIZE];
+  smpi_container(rank, str, INSTR_DEFAULT_STR_SIZE);
+  container_t container = PJ_container_get (str);
+  type_t type = PJ_type_get ("MPI_STATE", container->type);
+  const char *color = instr_find_color ("sleeping");
+  val_t value = PJ_value_get_or_new ("sleeping", color, type);
+  new_pajePushState (SIMIX_get_clock(), container, type, value);
+}
+
+void TRACE_smpi_sleeping_in(int rank, instr_extra_data extra)
+{
+  //do not forget to set the color first, otherwise this will explode
+  if (!TRACE_smpi_is_enabled()|| !TRACE_smpi_is_sleeping()) {
+      cleanup_extra_data(extra);
+      return;
+  }
+
+  char str[INSTR_DEFAULT_STR_SIZE];
+  smpi_container(rank, str, INSTR_DEFAULT_STR_SIZE);
+  container_t container = PJ_container_get (str);
+  type_t type = PJ_type_get ("MPI_STATE", container->type);
+  val_t value = PJ_value_get_or_new ("sleeping", NULL, type);
+  new_pajePushStateWithExtra  (SIMIX_get_clock(), container, type, value, (void*)extra);
+}
+
+void TRACE_smpi_sleeping_out(int rank)
+{
+  if (!TRACE_smpi_is_enabled()|| !TRACE_smpi_is_sleeping()) return;
+  char str[INSTR_DEFAULT_STR_SIZE];
+  smpi_container(rank, str, INSTR_DEFAULT_STR_SIZE);
+  container_t container = PJ_container_get (str);
+  type_t type = PJ_type_get ("MPI_STATE", container->type);
+  new_pajePopState (SIMIX_get_clock(), container, type);
+}
+
+
+void TRACE_smpi_testing_in(int rank, instr_extra_data extra)
+{
+  //do not forget to set the color first, otherwise this will explode
+  if (!TRACE_smpi_is_enabled()) {
+      cleanup_extra_data(extra);
+      return;
+  }
+
+  char str[INSTR_DEFAULT_STR_SIZE];
+  smpi_container(rank, str, INSTR_DEFAULT_STR_SIZE);
+  container_t container = PJ_container_get (str);
+  type_t type = PJ_type_get ("MPI_STATE", container->type);
+  val_t value = PJ_value_get_or_new ("test", NULL, type);
+  new_pajePushStateWithExtra  (SIMIX_get_clock(), container, type, value, (void*)extra);
+}
+
+void TRACE_smpi_testing_out(int rank)
+{
+  if (!TRACE_smpi_is_enabled()) return;
+  char str[INSTR_DEFAULT_STR_SIZE];
+  smpi_container(rank, str, INSTR_DEFAULT_STR_SIZE);
+  container_t container = PJ_container_get (str);
+  type_t type = PJ_type_get ("MPI_STATE", container->type);
+  new_pajePopState (SIMIX_get_clock(), container, type);
+}
+
 void TRACE_smpi_ptp_in(int rank, int src, int dst, const char *operation, instr_extra_data extra)
 {
   if (!TRACE_smpi_is_enabled()) {
