@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2013. The SimGrid Team.
+/* Copyright (c) 2007-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -226,8 +226,11 @@ static MPI_Request build_request(void *buf, int count,
   request->truncated = 0;
   request->real_size = 0;
   request->real_tag = 0;
+  if(flags & PERSISTENT)
+    request->refcount = 1;
+  else
+    request->refcount = 0;
 
-  request->refcount=1;
 #ifdef HAVE_TRACING
   request->send = 0;
   request->recv = 0;
@@ -257,8 +260,8 @@ void smpi_action_trace_run(char *path)
   action_fp=NULL;
   if (path) {
     action_fp = fopen(path, "r");
-    xbt_assert(action_fp != NULL, "Cannot open %s: %s", path,
-               strerror(errno));
+    if (action_fp == NULL)
+      xbt_die("Cannot open %s: %s", path, strerror(errno));
   }
 
   if (!xbt_dict_is_empty(action_queues)) {
@@ -290,7 +293,6 @@ MPI_Request smpi_mpi_send_init(void *buf, int count, MPI_Datatype datatype,
   MPI_Request request = NULL; /* MC needs the comm to be set to NULL during the call */
   request = build_request(buf==MPI_BOTTOM ? (void*)0 : buf, count, datatype, smpi_process_index(), smpi_group_index(smpi_comm_group(comm), dst), tag,
                           comm, PERSISTENT | SEND | PREPARED);
-  request->refcount++;
   return request;
 }
 
@@ -300,7 +302,6 @@ MPI_Request smpi_mpi_ssend_init(void *buf, int count, MPI_Datatype datatype,
   MPI_Request request = NULL; /* MC needs the comm to be set to NULL during the call */
   request = build_request(buf==MPI_BOTTOM ? (void*)0 : buf, count, datatype, smpi_process_index(), smpi_group_index(smpi_comm_group(comm), dst), tag,
                           comm, PERSISTENT | SSEND | SEND | PREPARED);
-  request->refcount++;
   return request;
 }
 
@@ -310,7 +311,6 @@ MPI_Request smpi_mpi_recv_init(void *buf, int count, MPI_Datatype datatype,
   MPI_Request request = NULL; /* MC needs the comm to be set to NULL during the call */
   request = build_request(buf==MPI_BOTTOM ? (void*)0 : buf, count, datatype, src == MPI_ANY_SOURCE ? MPI_ANY_SOURCE : smpi_group_index(smpi_comm_group(comm), src), smpi_process_index(), tag,
                           comm, PERSISTENT | RECV | PREPARED);
-  request->refcount++;
   return request;
 }
 
@@ -320,6 +320,9 @@ void smpi_mpi_start(MPI_Request request)
 
   xbt_assert(!request->action, "Cannot (re)start a non-finished communication");
   request->flags &= ~PREPARED;
+  request->flags &= ~FINISHED;
+  request->refcount++;
+
   if (request->flags & RECV) {
     print_request("New recv", request);
     //FIXME: if receive is posted with a large size, but send is smaller, mailboxes may not match !
@@ -331,7 +334,9 @@ void smpi_mpi_start(MPI_Request request)
     request->real_size=request->size;
     smpi_datatype_use(request->old_type);
     smpi_comm_use(request->comm);
-    request->action = simcall_comm_irecv(mailbox, request->buf, &request->real_size, &match_recv, request);
+    request->action = simcall_comm_irecv(mailbox, request->buf,
+                                         &request->real_size, &match_recv, &smpi_comm_copy_buffer_callback,
+                                         request, -1.0);
 
     //integrate pseudo-timing for buffering of small messages, do not bother to execute the simcall if 0
     double sleeptime = request->detached ? smpi_or(request->size) : 0.0;
@@ -362,15 +367,23 @@ void smpi_mpi_start(MPI_Request request)
       XBT_DEBUG("Send request %p is not in the permanent receive mailbox (buf: %p)",request,request->buf);
       mailbox = smpi_process_remote_mailbox(receiver);
     }
+
+    void* buf = request->buf;
     if ( (! (request->flags & SSEND)) && (request->size < sg_cfg_get_int("smpi/send_is_detached_thres"))) {
       void *oldbuf = NULL;
       request->detached = 1;
       request->refcount++;
       if(request->old_type->has_subtype == 0){
         oldbuf = request->buf;
-        if (oldbuf && request->size!=0){
-          request->buf = xbt_malloc(request->size);
-          memcpy(request->buf,oldbuf,request->size);
+        if (!_xbt_replay_is_active() && oldbuf && request->size!=0){
+          if((smpi_privatize_global_variables)
+      	      && ((char*)request->buf >= start_data_exe)
+	      && ((char*)request->buf < start_data_exe + size_data_exe )){
+            XBT_DEBUG("Privatization : We are sending from a zone inside global memory. Switch data segment ");
+       	    switch_data_segment(smpi_process_index());
+  	  }
+          buf = xbt_malloc(request->size);
+          memcpy(buf,oldbuf,request->size);
         }
       }
       XBT_DEBUG("Send request %p is detached; buf %p copied into %p",request,oldbuf,request->buf);
@@ -382,22 +395,23 @@ void smpi_mpi_start(MPI_Request request)
     smpi_comm_use(request->comm);
 
     //if we are giving back the control to the user without waiting for completion, we have to inject timings
-    double sleeptime =0.0;
+    double sleeptime = 0.0;
     if(request->detached || (request->flags & (ISEND|SSEND))){// issend should be treated as isend
       //isend and send timings may be different
       sleeptime = (request->flags & ISEND)? smpi_ois(request->size) : smpi_os(request->size);
     }
 
-    if(sleeptime!=0.0){
+    if(sleeptime != 0.0){
         simcall_process_sleep(sleeptime);
         XBT_DEBUG("sending size of %zu : sleep %f ", request->size, smpi_os(request->size));
     }
 
     request->action =
       simcall_comm_isend(mailbox, request->size, -1.0,
-                         request->buf, request->real_size,
+                         buf, request->real_size,
                          &match_send,
-                         &smpi_mpi_request_free_voidp, // how to free the userdata if a detached send fails
+                         &xbt_free, // how to free the userdata if a detached send fails
+                         &smpi_comm_copy_buffer_callback,
                          request,
                          // detach if msg size < eager/rdv switch limit
                          request->detached);
@@ -435,7 +449,6 @@ void smpi_mpi_request_free(MPI_Request * request)
         *request = MPI_REQUEST_NULL;
     }else{
         print_request("Decrementing", (*request));
-
     }
   }else{
       xbt_die("freeing an already free request");
@@ -447,7 +460,7 @@ MPI_Request smpi_isend_init(void *buf, int count, MPI_Datatype datatype,
 {
   MPI_Request request = NULL; /* MC needs the comm to be set to NULL during the call */
   request = build_request(buf==MPI_BOTTOM ? (void*)0 : buf , count, datatype, smpi_process_index(), smpi_group_index(smpi_comm_group(comm), dst), tag,
-                          comm, NON_PERSISTENT | ISEND | SEND | PREPARED);
+                          comm, PERSISTENT | ISEND | SEND | PREPARED);
   return request;
 }
 
@@ -478,7 +491,7 @@ MPI_Request smpi_irecv_init(void *buf, int count, MPI_Datatype datatype,
 {
   MPI_Request request = NULL; /* MC needs the comm to be set to NULL during the call */
   request = build_request(buf==MPI_BOTTOM ? (void*)0 : buf, count, datatype, src == MPI_ANY_SOURCE ? MPI_ANY_SOURCE : smpi_group_index(smpi_comm_group(comm), src), smpi_process_index(), tag,
-                          comm, NON_PERSISTENT | RECV | PREPARED);
+                          comm, PERSISTENT | RECV | PREPARED);
   return request;
 }
 
@@ -546,6 +559,8 @@ void smpi_mpi_sendrecv(void *sendbuf, int sendcount, MPI_Datatype sendtype,
     smpi_irecv_init(recvbuf, recvcount, recvtype, src, recvtag, comm);
   smpi_mpi_startall(2, requests);
   smpi_mpi_waitall(2, requests, stats);
+  smpi_mpi_request_free(&requests[0]);
+  smpi_mpi_request_free(&requests[1]);
   if(status != MPI_STATUS_IGNORE) {
     // Copy receive status
     *status = stats[1];
@@ -562,7 +577,8 @@ static void finish_wait(MPI_Request * request, MPI_Status * status)
   MPI_Request req = *request;
   smpi_empty_status(status);
 
-  if(!(req->detached && req->flags & SEND) && !(req->flags & PREPARED)){
+  if(!(req->detached && req->flags & SEND)
+      && !(req->flags & PREPARED)){
     if(status != MPI_STATUS_IGNORE) {
       int src = req->src == MPI_ANY_SOURCE ? req->real_src : req->src;
       status->MPI_SOURCE = smpi_group_rank(smpi_comm_group(req->comm), src);
@@ -578,6 +594,15 @@ static void finish_wait(MPI_Request * request, MPI_Status * status)
     MPI_Datatype datatype = req->old_type;
 
     if(datatype->has_subtype == 1){
+      if (!_xbt_replay_is_active()){
+        if( smpi_privatize_global_variables
+            && ((char*)req->old_buf >= start_data_exe)
+            && ((char*)req->old_buf < start_data_exe + size_data_exe )
+        ){
+            XBT_VERB("Privatization : We are unserializing to a zone in global memory - Switch data segment ");
+            switch_data_segment(smpi_process_index());
+        }
+      }
       // This part handles the problem of non-contignous memory
       // the unserialization at the reception
       s_smpi_subtype_t *subtype = datatype->substruct;
@@ -604,12 +629,12 @@ static void finish_wait(MPI_Request * request, MPI_Status * status)
   if(req->detached_sender!=NULL){
     smpi_mpi_request_free(&(req->detached_sender));
   }
-
-  if(req->flags & NON_PERSISTENT) {
-    smpi_mpi_request_free(request);
-  } else {
+  if(req->flags & PERSISTENT)
     req->action = NULL;
-  }
+  req->flags |= FINISHED;
+
+  smpi_mpi_request_free(request);
+
 }
 
 int smpi_mpi_test(MPI_Request * request, MPI_Status * status) {
@@ -623,6 +648,7 @@ int smpi_mpi_test(MPI_Request * request, MPI_Status * status) {
       flag = simcall_comm_test((*request)->action);
     if (flag) {
       finish_wait(request, status);
+      if (*request != MPI_REQUEST_NULL && !((*request)->flags & PERSISTENT))
       *request = MPI_REQUEST_NULL;
     }
   }
@@ -655,6 +681,7 @@ int smpi_mpi_testany(int count, MPI_Request requests[], int *index,
     if(i != -1) {
       *index = map[i];
       finish_wait(&requests[*index], status);
+      if (requests[*index] != MPI_REQUEST_NULL && (requests[*index]->flags & NON_PERSISTENT))
       requests[*index] = MPI_REQUEST_NULL;
       flag = 1;
     }
@@ -707,7 +734,7 @@ void smpi_mpi_probe(int source, int tag, MPI_Comm comm, MPI_Status* status){
 void smpi_mpi_iprobe(int source, int tag, MPI_Comm comm, int* flag, MPI_Status* status){
 
   MPI_Request request =build_request(NULL, 0, MPI_CHAR, source == MPI_ANY_SOURCE ? MPI_ANY_SOURCE : smpi_group_index(smpi_comm_group(comm), source), smpi_comm_rank(comm), tag,
-            comm, NON_PERSISTENT | RECV);
+            comm, PERSISTENT | RECV);
 
   //to avoid deadlock, we have to sleep some time here, or the timer won't advance and we will only do iprobe simcalls
   double sleeptime= sg_cfg_get_double("smpi/iprobe");
@@ -762,15 +789,15 @@ void smpi_mpi_wait(MPI_Request * request, MPI_Status * status)
 
   if ((*request)->action != NULL) { // this is not a detached send
     simcall_comm_wait((*request)->action, -1.0);
-  }
-
 #ifdef HAVE_MC
-  if(MC_is_active())
+  if(MC_is_active() && (*request)->action)
     (*request)->action->comm.dst_data = NULL; // dangling pointer : dst_data is freed with a wait, need to set it to NULL for system state comparison
 #endif
+  }
 
   finish_wait(request, status);
-  *request = MPI_REQUEST_NULL;
+  if (*request != MPI_REQUEST_NULL && ((*request)->flags & NON_PERSISTENT))
+      *request = MPI_REQUEST_NULL;
   // FIXME for a detached send, finish_wait is not called:
 }
 
@@ -789,7 +816,9 @@ int smpi_mpi_waitany(int count, MPI_Request requests[],
     size = 0;
     XBT_DEBUG("Wait for one of %d", count);
     for(i = 0; i < count; i++) {
-      if (requests[i] != MPI_REQUEST_NULL && !(requests[i]->flags & PREPARED)) {
+      if (requests[i] != MPI_REQUEST_NULL
+          && !(requests[i]->flags & PREPARED)
+          && !(requests[i]->flags & FINISHED)) {
         if (requests[i]->action != NULL) {
           XBT_DEBUG("Waiting any %p ", requests[i]);
           xbt_dynar_push(comms, &requests[i]->action);
@@ -800,6 +829,7 @@ int smpi_mpi_waitany(int count, MPI_Request requests[],
          size=0;//so we free the dynar but don't do the waitany call
          index=i;
          finish_wait(&requests[i], status);//cleanup if refcount = 0
+         if (requests[i] != MPI_REQUEST_NULL && (requests[i]->flags & NON_PERSISTENT))
          requests[i]=MPI_REQUEST_NULL;//set to null
          break;
          }
@@ -812,6 +842,7 @@ int smpi_mpi_waitany(int count, MPI_Request requests[],
       if (i != -1) {
         index = map[i];
         finish_wait(&requests[index], status);
+        if (requests[i] != MPI_REQUEST_NULL && (requests[i]->flags & NON_PERSISTENT))
         requests[index] = MPI_REQUEST_NULL;
       }
     }
@@ -845,6 +876,7 @@ int smpi_mpi_waitall(int count, MPI_Request requests[],
     }
   }
   for(c = 0; c < count; c++) {
+
     if (MC_is_active()) {
       smpi_mpi_wait(&requests[c], pstat);
       index = c;
@@ -852,6 +884,7 @@ int smpi_mpi_waitall(int count, MPI_Request requests[],
       index = smpi_mpi_waitany(count, requests, pstat);
       if (index == MPI_UNDEFINED)
         break;
+      if (requests[index] != MPI_REQUEST_NULL && (requests[index]->flags & NON_PERSISTENT))
       requests[index]=MPI_REQUEST_NULL;
     }
     if (status != MPI_STATUSES_IGNORE) {
@@ -881,6 +914,7 @@ int smpi_mpi_waitsome(int incount, MPI_Request requests[], int *indices,
       if(status != MPI_STATUSES_IGNORE) {
         status[index] = *pstat;
       }
+     if (requests[index] != MPI_REQUEST_NULL && (requests[index]->flags & NON_PERSISTENT))
      requests[index]=MPI_REQUEST_NULL;
     }else{
       return MPI_UNDEFINED;
@@ -906,8 +940,8 @@ int smpi_mpi_testsome(int incount, MPI_Request requests[], int *indices,
          if(status != MPI_STATUSES_IGNORE) {
            status[i] = *pstat;
          }
+         if ((requests[i] != MPI_REQUEST_NULL) && requests[i]->flags & NON_PERSISTENT)
          requests[i]=MPI_REQUEST_NULL;
-
       }
     }else{
       count_dead++;
@@ -964,6 +998,9 @@ void smpi_mpi_gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
     // Wait for completion of irecv's.
     smpi_mpi_startall(size - 1, requests);
     smpi_mpi_waitall(size - 1, requests, MPI_STATUS_IGNORE);
+    for(src = 0; src < size-1; src++) {
+      smpi_mpi_request_free(&requests[src]);
+    }
     xbt_free(requests);
   }
 }
@@ -1028,6 +1065,9 @@ void smpi_mpi_gatherv(void *sendbuf, int sendcount, MPI_Datatype sendtype,
     // Wait for completion of irecv's.
     smpi_mpi_startall(size - 1, requests);
     smpi_mpi_waitall(size - 1, requests, MPI_STATUS_IGNORE);
+    for(src = 0; src < size-1; src++) {
+      smpi_mpi_request_free(&requests[src]);
+    }
     xbt_free(requests);
   }
 }
@@ -1068,6 +1108,9 @@ void smpi_mpi_allgather(void *sendbuf, int sendcount,
   // Wait for completion of all comms.
   smpi_mpi_startall(2 * (size - 1), requests);
   smpi_mpi_waitall(2 * (size - 1), requests, MPI_STATUS_IGNORE);
+  for(other = 0; other < 2*(size-1); other++) {
+    smpi_mpi_request_free(&requests[other]);
+  }
   xbt_free(requests);
 }
 
@@ -1107,6 +1150,9 @@ void smpi_mpi_allgatherv(void *sendbuf, int sendcount,
   // Wait for completion of all comms.
   smpi_mpi_startall(2 * (size - 1), requests);
   smpi_mpi_waitall(2 * (size - 1), requests, MPI_STATUS_IGNORE);
+  for(other = 0; other < 2*(size-1); other++) {
+    smpi_mpi_request_free(&requests[other]);
+  }
   xbt_free(requests);
 }
 
@@ -1147,6 +1193,9 @@ void smpi_mpi_scatter(void *sendbuf, int sendcount, MPI_Datatype sendtype,
     // Wait for completion of isend's.
     smpi_mpi_startall(size - 1, requests);
     smpi_mpi_waitall(size - 1, requests, MPI_STATUS_IGNORE);
+    for(dst = 0; dst < size-1; dst++) {
+      smpi_mpi_request_free(&requests[dst]);
+    }
     xbt_free(requests);
   }
 }
@@ -1188,6 +1237,9 @@ void smpi_mpi_scatterv(void *sendbuf, int *sendcounts, int *displs,
     // Wait for completion of isend's.
     smpi_mpi_startall(size - 1, requests);
     smpi_mpi_waitall(size - 1, requests, MPI_STATUS_IGNORE);
+    for(dst = 0; dst < size-1; dst++) {
+      smpi_mpi_request_free(&requests[dst]);
+    }
     xbt_free(requests);
   }
 }
@@ -1250,6 +1302,8 @@ void smpi_mpi_reduce(void *sendbuf, void *recvbuf, int count,
       XBT_DEBUG("finished waiting any request with index %d", index);
       if(index == MPI_UNDEFINED) {
         break;
+      }else{
+        smpi_mpi_request_free(&requests[index]);
       }
       if(op) /* op can be MPI_OP_NULL that does nothing */
         smpi_op_apply(op, tmpbufs[index], recvbuf, &count, &datatype);
@@ -1335,6 +1389,9 @@ void smpi_mpi_scan(void *sendbuf, void *recvbuf, int count,
   for(index = 0; index < rank; index++) {
     xbt_free(tmpbufs[index]);
   }
+  for(index = 0; index < size-1; index++) {
+    smpi_mpi_request_free(&requests[index]);
+  }
   xbt_free(tmpbufs);
   xbt_free(requests);
 }
@@ -1404,6 +1461,9 @@ void smpi_mpi_exscan(void *sendbuf, void *recvbuf, int count,
   for(index = 0; index < rank; index++) {
     xbt_free(tmpbufs[index]);
   }
+  for(index = 0; index < size-1; index++) {
+    smpi_mpi_request_free(&requests[index]);
+  }
   xbt_free(tmpbufs);
   xbt_free(requests);
 }
