@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2012. The SimGrid Team.
+/* Copyright (c) 2007-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -122,6 +122,7 @@ int ns3_initialize(const char* TcpProtocol){
   }
 
   XBT_ERROR("The ns3/TcpModel must be : NewReno or Reno or Tahoe");
+  return 0;
 }
 
 void * ns3_add_host(const char * id)
@@ -167,7 +168,7 @@ void * ns3_add_router(const char * id)
 	return router;
 }
 
-void * ns3_add_cluster(char * bw,char * lat,const char *id)
+void ns3_add_cluster(char * bw,char * lat,const char *id)
 {
 
 	XBT_DEBUG("cluster_id: %s",id);
@@ -175,7 +176,7 @@ void * ns3_add_cluster(char * bw,char * lat,const char *id)
 	XBT_DEBUG("Number of %s nodes: %d",id,Cluster_nodes.GetN() - number_of_clusters_nodes);
 
 	NodeContainer Nodes;
-	int i;
+	unsigned int i;
 
 	for(i = number_of_clusters_nodes; i < Cluster_nodes.GetN() ; i++){
 		Nodes.Add(Cluster_nodes.Get(i));
@@ -225,7 +226,7 @@ static char* transformIpv4Address (Ipv4Address from){
 		return bprintf("%s",s.c_str());
 }
 
-void * ns3_add_link(int src, e_ns3_network_element_type_t type_src,
+void ns3_add_link(int src, e_ns3_network_element_type_t type_src,
 					int dst, e_ns3_network_element_type_t type_dst,
 					char * bw,char * lat)
 {
@@ -234,7 +235,6 @@ void * ns3_add_link(int src, e_ns3_network_element_type_t type_src,
 		LogComponentEnable("UdpEchoServerApplication", LOG_LEVEL_INFO);
 	}
 
-
 	MyPointToPointHelper pointToPoint;
 
 	NetDeviceContainer netA;
@@ -274,7 +274,7 @@ void * ns3_add_link(int src, e_ns3_network_element_type_t type_src,
 	}
 }
 
-void * ns3_end_platform(void)
+void ns3_end_platform(void)
 {
 	XBT_DEBUG("InitializeRoutes");
 	GlobalRouteManager::BuildGlobalRoutingDatabase();
