@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2013. The SimGrid Team.
+/* Copyright (c) 2006-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -14,6 +14,7 @@
 #include "xbt/dict.h"
 #include "surf/surfxml_parse.h"
 #include "surf/surf_private.h"
+#include "surf/random_mgr.h"
 #include "simgrid/sg_config.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_parse, surf,
@@ -26,7 +27,6 @@ int ETag_surfxml_include_state(void);
 char* surf_parsed_filename = NULL; // to locate parse error messages
 
 xbt_dynar_t parsed_link_list = NULL;   /* temporary store of current list link of a route */
-extern AS_t current_routing;
 /*
  * Helping functions
  */
@@ -202,6 +202,7 @@ double surf_parse_get_power(const char *string)
 
 /* The default current property receiver. Setup in the corresponding opening callbacks. */
 xbt_dict_t current_property_set = NULL;
+xbt_dict_t current_model_property_set = NULL;
 xbt_dict_t as_current_property_set = NULL;
 int AS_TAG = 0;
 char* as_name_tab[1024];
@@ -215,9 +216,6 @@ xbt_dict_t random_data_list = NULL;
 YY_BUFFER_STATE surf_input_buffer;
 FILE *surf_file_to_parse = NULL;
 
-static void init_randomness(void);
-static void add_randomness(void);
-
 /*
  * Stuff relative to storage
  */
@@ -237,6 +235,7 @@ void ETag_surfxml_storage(void)
   storage.content = A_surfxml_storage_content;
   storage.content_type = A_surfxml_storage_content___type;
   storage.properties = current_property_set;
+  storage.attach = A_surfxml_storage_attach;
   sg_platf_new_storage(&storage);
   current_property_set = NULL;
 }
@@ -245,6 +244,7 @@ void STag_surfxml_storage___type(void)
   AS_TAG = 0;
   XBT_DEBUG("STag_surfxml_storage___type");
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
+  xbt_assert(current_model_property_set == NULL, "Someone forgot to reset the model property set to NULL in its closing tag (or XML malformed)");
 }
 void ETag_surfxml_storage___type(void)
 {
@@ -256,9 +256,11 @@ void ETag_surfxml_storage___type(void)
   storage_type.id = A_surfxml_storage___type_id;
   storage_type.model = A_surfxml_storage___type_model;
   storage_type.properties = current_property_set;
+  storage_type.model_properties = current_model_property_set;
   storage_type.size = surf_parse_get_size(A_surfxml_storage___type_size);
   sg_platf_new_storage_type(&storage_type);
   current_property_set = NULL;
+  current_model_property_set = NULL;
 }
 void STag_surfxml_mstorage(void)
 {
@@ -296,6 +298,7 @@ static xbt_dynar_t surf_parsed_filename_stack = NULL;
 
 void STag_surfxml_include(void)
 {
+  parse_after_config();
   XBT_DEBUG("STag_surfxml_include '%s'",A_surfxml_include_file);
   xbt_dynar_push(surf_parsed_filename_stack,&surf_parsed_filename); // save old file name
   surf_parsed_filename = xbt_strdup(A_surfxml_include_file);
@@ -417,8 +420,8 @@ void STag_surfxml_prop(void)
   }
   else{
     if (!current_property_set)
-      current_property_set = xbt_dict_new_homogeneous(xbt_free_f); // Maybe, it should raise an error
-    xbt_dict_set(current_property_set, A_surfxml_prop_id, xbt_strdup(A_surfxml_prop_value), NULL);
+       current_property_set = xbt_dict_new(); // Maybe, it should raise an error
+    xbt_dict_set(current_property_set, A_surfxml_prop_id, xbt_strdup(A_surfxml_prop_value), xbt_free);
   }
 }
 
@@ -519,6 +522,23 @@ void ETag_surfxml_cluster(void){
     cluster.loopback_bw = surf_parse_get_bandwidth(A_surfxml_cluster_loopback___bw);
   if(strcmp(A_surfxml_cluster_loopback___lat,""))
     cluster.loopback_lat = surf_parse_get_time(A_surfxml_cluster_loopback___lat);
+
+  switch(AX_surfxml_cluster_topology){
+  case A_surfxml_cluster_topology_FLAT:
+    cluster.topology= SURF_CLUSTER_FLAT ;
+    break;
+  case A_surfxml_cluster_topology_TORUS:
+    cluster.topology= SURF_CLUSTER_TORUS ;
+    break;
+  case A_surfxml_cluster_topology_FAT___TREE:
+    cluster.topology = SURF_CLUSTER_FAT_TREE;
+    break;
+  default:
+    surf_parse_error("Invalid cluster topology for cluster %s",
+                     cluster.id);
+    break;
+  }
+  cluster.topo_parameters = A_surfxml_cluster_topo___parameters;
   cluster.router_id = A_surfxml_cluster_router___id;
 
   switch (AX_surfxml_cluster_sharing___policy) {
@@ -552,15 +572,17 @@ void ETag_surfxml_cluster(void){
   cluster.availability_trace = A_surfxml_cluster_availability___file;
   cluster.state_trace = A_surfxml_cluster_state___file;
   sg_platf_new_cluster(&cluster);
-  
+
   current_property_set = NULL;
 }
 
-void STag_surfxml_cluster(void){ 
+void STag_surfxml_cluster(void){
+  parse_after_config();
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
 
 void STag_surfxml_cabinet(void){
+  parse_after_config();
   s_sg_platf_cabinet_cbarg_t cabinet;
   memset(&cabinet,0,sizeof(cabinet));
   cabinet.id = A_surfxml_cabinet_id;
@@ -575,6 +597,7 @@ void STag_surfxml_cabinet(void){
 }
 
 void STag_surfxml_peer(void){
+  parse_after_config();
   s_sg_platf_peer_cbarg_t peer;
   memset(&peer,0,sizeof(peer));
   peer.id = A_surfxml_peer_id;
@@ -749,6 +772,13 @@ void ETag_surfxml_ASroute(void){
   ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw___src);
   ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw___dst);
 
+  if (A_surfxml_ASroute_gw___src && !ASroute.gw_src)
+    surf_parse_error("gw_src=\"%s\" not found for ASroute from \"%s\" to \"%s\"",
+                     A_surfxml_ASroute_gw___src, ASroute.src, ASroute.dst);
+  if (A_surfxml_ASroute_gw___dst && !ASroute.gw_dst)
+    surf_parse_error("gw_dst=\"%s\" not found for ASroute from \"%s\" to \"%s\"",
+                     A_surfxml_ASroute_gw___dst, ASroute.src, ASroute.dst);
+
   ASroute.link_list = parsed_link_list;
 
   switch (A_surfxml_ASroute_symmetrical) {
@@ -809,6 +839,7 @@ void ETag_surfxml_trace(void){
 }
 
 void STag_surfxml_trace___connect(void){
+  parse_after_config();
   s_sg_platf_trace_connect_cbarg_t trace_connect;
   memset(&trace_connect,0,sizeof(trace_connect));
 
@@ -837,6 +868,7 @@ void STag_surfxml_trace___connect(void){
 }
 
 void STag_surfxml_AS(void){
+  parse_after_config();
   AS_TAG = 1;
   s_sg_platf_AS_cbarg_t AS = SG_PLATF_AS_INITIALIZER;
   AS.id = A_surfxml_AS_id;
@@ -883,6 +915,7 @@ void ETag_surfxml_config(void){
     free(cfg);
   }
   XBT_DEBUG("End configuration name = %s",A_surfxml_config_id);
+
   xbt_dict_free(&current_property_set);
   current_property_set = NULL;
 }
@@ -930,6 +963,13 @@ void STag_surfxml_argument(void){
   argv[(argc) - 1] = xbt_strdup(A_surfxml_argument_value);
 }
 
+void STag_surfxml_model___prop(void){
+  if (!current_model_property_set)
+    current_model_property_set = xbt_dict_new_homogeneous(xbt_free_f);
+
+  xbt_dict_set(current_model_property_set, A_surfxml_model___prop_id, xbt_strdup(A_surfxml_model___prop_value), NULL);
+}
+
 /* ***************************************** */
 /* TUTORIAL: New TAG                         */
 void STag_surfxml_gpu(void)
@@ -952,7 +992,7 @@ void ETag_surfxml_prop(void){}
 void STag_surfxml_random(void){}
 void ETag_surfxml_random(void){}
 void ETag_surfxml_trace___connect(void){}
-void STag_surfxml_trace(void){}
+void STag_surfxml_trace(void){parse_after_config();}
 void ETag_surfxml_router(void){}
 void ETag_surfxml_host___link(void){}
 void ETag_surfxml_cabinet(void){}
@@ -960,6 +1000,7 @@ void ETag_surfxml_peer(void){}
 void STag_surfxml_backbone(void){}
 void ETag_surfxml_link___ctn(void){}
 void ETag_surfxml_argument(void){}
+void ETag_surfxml_model___prop(void){}
 
 /* Open and Close parse file */
 
@@ -1022,8 +1063,6 @@ int_f_void_t surf_parse = _surf_parse;
  * With XML parser
  */
 
-/* Random tag functions */
-
 double get_cpu_power(const char *power)
 {
   double power_scale = 0.0;
@@ -1051,44 +1090,6 @@ double random_min, random_max, random_mean, random_std_deviation;
 e_random_generator_t random_generator;
 char *random_id;
 
-static void init_randomness(void)
-{
-  random_id = A_surfxml_random_id;
-  random_min = surf_parse_get_double(A_surfxml_random_min);
-  random_max = surf_parse_get_double(A_surfxml_random_max);
-  random_mean = surf_parse_get_double(A_surfxml_random_mean);
-  random_std_deviation = surf_parse_get_double(A_surfxml_random_std___deviation);
-  switch (A_surfxml_random_generator) {
-  case AU_surfxml_random_generator:
-  case A_surfxml_random_generator_NONE:
-    random_generator = NONE;
-    break;
-  case A_surfxml_random_generator_DRAND48:
-    random_generator = DRAND48;
-    break;
-  case A_surfxml_random_generator_RAND:
-    random_generator = RAND;
-    break;
-  case A_surfxml_random_generator_RNGSTREAM:
-    random_generator = RNGSTREAM;
-    break;
-  default:
-    surf_parse_error("Invalid random generator");
-    break;
-  }
-}
-
-static void add_randomness(void)
-{
-  /* If needed, additional properties can be added by using the prop tag */
-  random_data_t random =
-      random_new(random_generator, 0, random_min, random_max, random_mean,
-                 random_std_deviation);
-  xbt_dict_set(random_data_list, random_id, (void *) random,
-               &xbt_free_ref);
-}
-
-
 xbt_dict_t get_as_router_properties(const char* name)
 {
   return xbt_lib_get_or_null(as_router_lib, name, ROUTING_PROP_ASR_LEVEL);
