@@ -1,12 +1,11 @@
 /* log - a generic logging facility in the spirit of log4j                  */
 
-/* Copyright (c) 2004-2013. The SimGrid Team.
+/* Copyright (c) 2004-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-
 #include <stdarg.h>
 #include <ctype.h>
 #include <stdio.h>              /* snprintf */
@@ -421,6 +420,16 @@ The following example resets the additivity of the xbt category to true (which i
     option and specify the max file size. This would be a nice default for
     non-kernel applications.
   - Careful, category names are global variables.
+  - When writing a log format, you often want to use spaces. If you don't
+    protect these spaces, they are used as configuration elements separators.
+    For example, if you want to remove the date from the logs, you want to pass the following 
+    argument on the command line. The outer quotes are here to protect the string from the shell 
+    interpretation while the inner ones are there to prevent simgrid from splitting the string 
+    in several log parameters (that would be invalid).
+    \verbatim --log="'root.fmt:%l: [%p/%c]: %m%n'"\endverbatim
+    Another option is to use the SimGrid-specific format directive \%e for
+    spaces, like in the following.
+    \verbatim --log="root.fmt:%l:%e[%p/%c]:%e%m%n"\endverbatim
 
 \section log_internals 4. Internal considerations
 
@@ -546,7 +555,7 @@ static void xbt_log_connect_categories(void)
   XBT_LOG_CONNECT(xbt);
   XBT_LOG_CONNECT(graphxml_parse);
   XBT_LOG_CONNECT(log);
-#if HAVE_MMAP
+#if HAVE_MMALLOC
   XBT_LOG_CONNECT(mm_diff);
 #endif
   XBT_LOG_CONNECT(module);
@@ -624,7 +633,9 @@ static void xbt_log_connect_categories(void)
   XBT_LOG_CONNECT(mc_checkpoint);
   XBT_LOG_CONNECT(mc_compare);
   XBT_LOG_CONNECT(mc_dpor);
+  XBT_LOG_CONNECT(mc_dwarf);
   XBT_LOG_CONNECT(mc_global);
+  XBT_LOG_CONNECT(mc_hash);
   XBT_LOG_CONNECT(mc_liveness);
   XBT_LOG_CONNECT(mc_memory);
   XBT_LOG_CONNECT(mc_memory_map);
@@ -667,6 +678,7 @@ static void xbt_log_connect_categories(void)
   XBT_LOG_CONNECT(simix_process);
   XBT_LOG_CONNECT(simix_smurf);
   XBT_LOG_CONNECT(simix_synchro);
+  XBT_LOG_CONNECT(simix_vm);
 
   /* smpi */
   /* SMPI categories are connected in smpi_global.c */
@@ -677,13 +689,14 @@ static void xbt_log_connect_categories(void)
   XBT_LOG_CONNECT(random);
   XBT_LOG_CONNECT(surf_config);
   XBT_LOG_CONNECT(surf_cpu);
+  XBT_LOG_CONNECT(surf_cpu_cas);
   XBT_LOG_CONNECT(surf_cpu_ti);
+  XBT_LOG_CONNECT(surf_energy);
   XBT_LOG_CONNECT(surf_kernel);
   XBT_LOG_CONNECT(surf_lagrange);
   XBT_LOG_CONNECT(surf_lagrange_dichotomy);
   XBT_LOG_CONNECT(surf_maxmin);
   XBT_LOG_CONNECT(surf_network);
-  XBT_LOG_CONNECT(surf_new_model);
 #ifdef HAVE_GTNETS
   XBT_LOG_CONNECT(surf_network_gtnets);
   XBT_LOG_CONNECT(surf_network_gtnets_interface);
@@ -699,13 +712,16 @@ static void xbt_log_connect_categories(void)
   XBT_LOG_CONNECT(surf_route);
   XBT_LOG_CONNECT(surf_routing_generic);
   XBT_LOG_CONNECT(surf_route_cluster);
+  XBT_LOG_CONNECT(surf_route_cluster_torus);
   XBT_LOG_CONNECT(surf_route_dijkstra);
+  XBT_LOG_CONNECT(surf_route_fat_tree);
   XBT_LOG_CONNECT(surf_route_floyd);
   XBT_LOG_CONNECT(surf_route_full);
   XBT_LOG_CONNECT(surf_route_none);
   XBT_LOG_CONNECT(surf_route_vivaldi);
   XBT_LOG_CONNECT(surf_storage);
   XBT_LOG_CONNECT(surf_trace);
+  XBT_LOG_CONNECT(surf_vm_workstation);
   XBT_LOG_CONNECT(surf_workstation);
 
 #endif /* simgrid_EXPORTS */
@@ -1059,7 +1075,7 @@ void xbt_log_threshold_set(xbt_log_category_t cat,
 
 static xbt_log_setting_t _xbt_log_parse_setting(const char *control_string)
 {
-
+  const char *orig_control_string = control_string;
   xbt_log_setting_t set = xbt_new(s_xbt_log_setting_t, 1);
   const char *name, *dot, *eq;
 
@@ -1082,7 +1098,7 @@ static xbt_log_setting_t _xbt_log_parse_setting(const char *control_string)
   control_string += strcspn(control_string, " ");
 
   xbt_assert(*dot == '.' && (*eq == '=' || *eq == ':'),
-              "Invalid control string '%s'", control_string);
+              "Invalid control string '%s'", orig_control_string);
 
   if (!strncmp(dot + 1, "threshold", (size_t) (eq - dot - 1))) {
     int i;
