@@ -93,9 +93,9 @@ int builtin_float_test(void)
 {
     int nints, nadds, ntypes, combiner;
 
-    int err, errs = 0;
+    int /* err, */ errs = 0;
 
-    err = MPI_Type_get_envelope(MPI_FLOAT,
+    /* err = */ MPI_Type_get_envelope(MPI_FLOAT,
 				&nints,
 				&nadds,
 				&ntypes,
@@ -324,17 +324,17 @@ int optimizable_vector_of_basics_test(void)
     MPI_Aint *adds = NULL;
     MPI_Datatype *types;
 
-    int err, errs = 0;
+    int /* err, */ errs = 0;
 
     /* set up type */
-    err = MPI_Type_vector(10,
+    /* err = */ MPI_Type_vector(10,
 			  2,
 			  2,
 			  MPI_INT,
 			  &parent_type);
 
     /* decode */
-    err = MPI_Type_get_envelope(parent_type,
+    /* err = */ MPI_Type_get_envelope(parent_type,
 				&nints,
 				&nadds,
 				&ntypes,
@@ -358,7 +358,7 @@ int optimizable_vector_of_basics_test(void)
     if (nadds) adds = malloc(nadds * sizeof(*adds));
     types = malloc(ntypes *sizeof(*types));
 
-    err = MPI_Type_get_contents(parent_type,
+    /* err = */ MPI_Type_get_contents(parent_type,
 				nints,
 				nadds,
 				ntypes,
@@ -407,17 +407,17 @@ int indexed_of_basics_test(void)
     MPI_Aint *adds = NULL;
     MPI_Datatype *types;
 
-    int err, errs = 0;
+    int /* err, */ errs = 0;
 
     /* set up type */
-    err = MPI_Type_indexed(s_count,
+    /* err = */ MPI_Type_indexed(s_count,
 			   s_blocklengths,
 			   s_displacements,
 			   MPI_INT,
 			   &parent_type);
 
     /* decode */
-    err = MPI_Type_get_envelope(parent_type,
+    /* err = */ MPI_Type_get_envelope(parent_type,
 				&nints,
 				&nadds,
 				&ntypes,
@@ -441,7 +441,7 @@ int indexed_of_basics_test(void)
     if (nadds) adds = malloc(nadds * sizeof(*adds));
     types = malloc(ntypes *sizeof(*types));
 
-    err = MPI_Type_get_contents(parent_type,
+    /* err = */ MPI_Type_get_contents(parent_type,
 				nints,
 				nadds,
 				ntypes,
