@@ -1,6 +1,6 @@
 /* signal -- what TESH needs to know about signals                          */
 
-/* Copyright (c) 2007-2010, 2012. The SimGrid Team.
+/* Copyright (c) 2007-2010, 2012-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -48,7 +48,7 @@ static const s_signal_entry_t signals[] = {
 };
 
 
-const char *signal_name(unsigned int got, char *expected)
+const char *signal_name(int got, char *expected)
 {
   int i;
 
