@@ -1,6 +1,6 @@
 /* TESH (Test Shell) -- mini shell specialized in running test units        */
 
-/* Copyright (c) 2007-2011. The SimGrid Team.
+/* Copyright (c) 2007-2011, 2013-2014. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -17,7 +17,7 @@
 /******************************************/
 /* return the name of a signal, aliasing SIGBUS to SIGSEGV since
    segfault leads to any of them depending on the system */
-const char *signal_name(unsigned int got, char *expected);
+const char *signal_name(int got, char *expected);
 
 #include "run_context.h"
 
